/*
 * This macro allows you to use a 'C' function to handle an 8051 interrupt.
 *
 * The only parameter is the decimal address of the interrupt vector.
 * Defines are provided for the common values.
 *
 * The macro MUST be placed IMMEDIATELY preceeding the definition of the
 * function which is to be the interrupt handler. Example:
 *
 * 		INTERRUPT(_SER_) serial_int_handler()
 * 		{
 *			/* ... Function code to handle serial interrupt ... */
 *		}
 *
 *
 * Since this is a parameterized multi-line macro, the MCP extended pre-
 * processor MUST be used. An alternative to this macro which does not
 * require MCP is to create "canned" include files for each interrupt
 * vector, and #include them before the function declarations.
 *
 * BEFORE USING THIS MACRO... Make sure that the startup code leaves
 * room for the interrupt vectors ($0003 - $0032), Use something like
 * this at the beginning of the "prefix" file:
 *
 *					AJMP	*+$0032		Jump around vectors
 *					DS		$0032-2		Reserve space for vectors
 *
 * Unfortunately, the code generated by the compiler and in the runtime
 * library sometimes has to reference registers by address. This means
 * that we cannot simply "switch" the register bank during the interrupt,
 * but have to "push/pop" all of the registers instead (Yech!).
 *
 * Copyright 1991,1993 Dave Dunfield
 * All rights reserved.
 */
#define	INTERRUPT(vec) asm {\
$SE:0						/* Insure output in code segment */\
_/**/vec	/* This label will be the address of the "prefix" stub */\
		ORG		vec			/* Position to interrupt vector */\
		PUSH	PSW			/* Save Processor Status Word */\
		PUSH	A			/* Save accumulator (2 bytes) */\
		LJMP	_/**/vec	/* Don't overwrite next vector (+3 = 7) */\
		ORG		_/**/vec	/* Position back to original PC */\
		PUSH	B			/* Save B register (+2 = 4) */\
		PUSH	DPH			/* Save Data Pointer (HIGH)*/\
		PUSH	DPL			/* Save Data Pointer (LOW)*/\
		PUSH	0			/* Save R0 */\
		PUSH	1			/* Save R1 */\
		PUSH	2			/* Save R2 */\
		PUSH	3			/* Save R3 */\
		PUSH	4			/* Save R4 */\
		PUSH	5			/* Save R5 */\
		PUSH	6			/* Save R6 */\
		PUSH	7			/* Save R7 */\
		LCALL	_/**/vec+52	/* Invoke 'C' interrupt handler */\
		POP		7			/* Restore R7 */\
		POP		6			/* Restore R6 */\
		POP		5			/* Restore R5 */\
		POP		4			/* Restore R4 */\
		POP		3			/* Restore R3 */\
		POP		2			/* Restore R2 */\
		POP		1			/* Restore R1 */\
		POP		0			/* Restore R0 */\
		POP		DPL			/* Restore Data Pointer (LOW) */\
		POP		DPH			/* Restore Data Pointer (HIGH) */\
		POP		B			/* Restore B register */\
		POP		A			/* Restore A accumulator */\
		POP		PSW			/* Restore Processor Status Word */\
		RETI				/* End interrupt */\
}

/*
 * The following interrupt definitions are sutible for use when placing
 * you code directly in ROM at address 0 in the 8051 memory map. When
 * running under a debugger etc. which re-directs interrupts, you will
 * have to modify the values to the new addresses.
 */
#define	_IE0_	3		/* External interrupt 0 */
#define	_TF0_	11		/* Timer 0 rollover */
#define	_IE1_	19		/* External interrupt 1 */
#define	_TF1_	27		/* Timer 1 rollover */
#define	_SER_	35		/* Serial port RX or TX */
