/*
 * This program demonstrates how to use various features of the 8051
 * MICRO-C implementation. It is intended for examination only, and is
 * unlikely to perform any useful function if you run it (it accesses
 * non-existant hardware).
 *
 * Compile with the EXTENDED PRE-PROCESSOR ('-p' option), in any memory
 * model EXCEPT tiny.
 */
#include <8051io.h>		/* General I/O definitions */
#include <8051bit.h>	/* Bit set/clear macros */
#include <8051reg.h>	/* 8051 register definitions */

/*
 * Defining global variables
 */

/* Define an array in EXTERNAL memory */
	int Earray[10];

/* Define an array in INTERNAL memory */
	register int Iarray[10];

/*
 * Accessing internal .vs. external memory
 *
 * Note, we are using GLOBAL arrays here, so that we can be
 * sure of their location (internal/external). When using
 * LOCAL varaibles, they will be placed on a "stack" in:
 *  TINY, SMALL, MEDIUM   = INTERNAL Ram
 *  COMPACT, LARGE        = EXTERNAL Ram
 */
demo1()
{
	int *Eptr;			/* Points to EXTERNAL memory */
	register int *Iptr;	/* Points to INTERNAL memory */
	int i;				/* General counter */

	/* Init arrays using indexing */
	for(i=0; i < 10; ++i)
		Earray[i] = Iarray[i] = i;

	/* Dump arrays using pointers */
	Eptr = Earray;		/* To reference EXTERNAL memory */
	Iptr = Iarray;		/* To reference INTERNAL memory */
	for(i=0; i < 10; ++i)
		printf("Earray[%u] = %u - Iarray[%u] = %u\n", i, *Eptr++, i, *Iptr++);
}

/*
 * Testing and setting bits...
 *
 * Wait for P1.0 to go high, and then toggle P1.1 until P1.0 goes LOW again.
 *
 * If you use the optimizer ('-o' option), the 'while' statements will be
 * recognized as bit testing, and converted into direct 'JB' and 'JNB'
 * instructions. The 'setbit' and 'clrbit' macros each generate a single
 * 'SETB' or 'CLRB' instruction.
 *
 * NOTE that 'setbit' and 'clrbit' expand into a 'asm { ... }' construct,
 * which constitutes a single MICRO-C statement. Since the 'if' statement
 * does not use '{ }' to allow multiple statements, a ';' following the
 * 'setbit' macro would be considered a second statement, and cause the
 * parser to assume that there is no 'else' clause. This would cause an
 * error when the else is encountered on the next line.
 */
demo2()
{
	int i;
	i = 0;
	while(!(P1 & 0x01))		/* Wait for P1 to come high */
		;
	while(P1 & 0x01) {
		if(++i >= 0)
			setbit(P1.1)	/* MACRO's don't use ';' - See NOTE */
		else
			clrbit(P1.2)	/* MACRO's don't use ';' - See NOTE */
	};

}

/*
 * Accessing a 'constant' external memory location. (For access to
 * external hardware etc.)
 *
 * Note: in the TINY model, the only way to read/write the external
 * RAM space is with the 'peek' and 'poke' functions from the
 * library. (Or use INLINE ASSEMBLER)
 */
#define	EXT_HW_CTL *(char*)0xF800		/* Control register at 0xF800 */
#define	EXT_HW_DAT *(char*)0xF801		/* Data    register at 0xF801 */
demo3()
{
	while(EXT_HW_CTL & 0x01)		/* Wait for Bit to go low */
		;
	EXT_HW_DAT = 0xff;				/* Write a byte (0xff) */
	EXT_HW_CTL |= 0x02;				/* Set output bit */
}

main()
{
	demo1();
	demo2();
	demo2();
}
